import numpy

from matrix import Matrix


class Vector(Matrix):
    def __init__(self, data):
        """

        :type data: numpy.ndarray | list | int
        """
        if isinstance(data, numpy.ndarray):
            super().__init__(data)
        elif isinstance(data, list):
            super().__init__(data, (len(data), 1))
        elif isinstance(data, int):
            self.__init__([0] * data)
        else:
            raise ValueError("data must be a list or an integer for dimension")

    def get_dimension(self):
        return super().shape()[0]

    def transpose(self):
        return Vector(self.__data__.reshape(self.__shape__[1], self.__shape__[0]))

    def __mul__(self, other):
        if isinstance(other, Vector):
            if self.shape() == other.shape():
                return Vector(self.__data__ * other.__data__)
            return super().__mul__(other)[0][0]
        elif isinstance(other, int) or isinstance(other, float):
            return Vector(super().__mul__(other).__data__)
        else:
            raise ValueError("A vector can only be multiplied with an vector (dot product) or a scalar")

    def __rmul__(self, other):
        return self * other

    def norm(self, **kwargs):
        """
        Computes the 2-norm of the vector which is the Frobenius-Norm of a nx1 matrix.

        :param kwargs: ignored
        :return: the 2-norm of the vector
        """
        return super().norm()

    def normalize(self):
        """
        A normalized vector has the length (norm) 1.
        To achieve that the vector is divided by the norm of itself.

        :return: the normalized vector
        """
        return self / self.norm()
